@@ -6,21 +6,19 @@ by hand.
 
 """
 import ast
+from inspect import isfunction, ismethod
 
 from parsimonious.exceptions import UndefinedLabel
 from parsimonious.expressions import (Literal, Regex, Sequence, OneOf,
-    Lookahead, Optional, ZeroOrMore, OneOrMore, Not)
+    Lookahead, Optional, ZeroOrMore, OneOrMore, Not, expression)
 from parsimonious.nodes import NodeVisitor
 from parsimonious.utils import StrAndRepr
 
 
-__all__ = ['Grammar']
-
-
 class Grammar(StrAndRepr, dict):
-    """A collection of expressions that describe a language
+    """A collection of rules that describe a language
 
-    You can start parsing from the default expression by calling ``parse()``
+    You can start parsing from the default rule by calling ``parse()``
     directly on the ``Grammar`` object::
 
         g = Grammar('''
@@ -30,10 +28,10 @@ class Grammar(StrAndRepr, dict):
                     ''')
         g.parse('Hello, my good sir')
 
-    Or start parsing from any of the other expressions; you can pull them out
-    of the grammar as if it were a dictionary::
+    Or start parsing from any of the other rules; you can pull them out of the
+    grammar as if it were a dictionary::
 
-        g['greeting'].parse('Hi')
+        g['title'].parse('sir')
 
     You could also just construct a bunch of ``Expression`` objects yourself
     and stitch them together into a language, but using a ``Grammar`` has some
@@ -46,26 +44,49 @@ class Grammar(StrAndRepr, dict):
       increase cache hit ratio. [Is this implemented yet?]
 
     """
-    def __init__(self, rules, default_rule=None):
+    def __init__(self, rules='', **more_rules):
         """Construct a grammar.
 
-        :arg rules: A string of production rules, one per line. There must be
-            at least one rule.
+        :arg rules: A string of production rules, one per line.
         :arg default_rule: The name of the rule invoked when you call
-            ``parse()`` on the grammar. Defaults to the first rule.
+            :meth:`parse()` or :meth:`match()` on the grammar. Defaults to the
+            first rule. Falls back to None if there are no string-based rules
+            in this grammar.
+        :arg more_rules: Additional kwargs whose names are rule names and
+            values are Expressions or custom-coded callables which accomplish
+            things the built-in rule syntax cannot. These take precedence over
+            ``rules`` in case of naming conflicts.
 
         """
-        # We can either have extending callers pull the rule text out of repr,
-        # or we could get fancy and define __add__ on Grammars and strings. Or
-        # maybe, if you want to extend a grammar, just prepend (or append?)
-        # your string to its, and yours will take precedence. Or use the OMeta
-        # delegation syntax.
-        exprs, first = self._expressions_from_rules(rules)
+        decorated_custom_rules = dict(
+            (k, expression(v, k, self) if isfunction(v) or
+                                          ismethod(v) else
+                v) for k, v in more_rules.iteritems())
+
+        exprs, first = self._expressions_from_rules(rules, decorated_custom_rules)
 
         self.update(exprs)
-        self.default_rule = exprs[default_rule] if default_rule else first
+        self.default_rule = first  # may be None
 
-    def _expressions_from_rules(self, rules):
+    def default(self, rule_name):
+        """Return a new Grammar whose :term:`default rule` is ``rule_name``."""
+        new = self._copy()
+        new.default_rule = new[rule_name]
+        return new
+
+    def _copy(self):
+        """Return a shallow copy of myself.
+
+        Deep is unnecessary, since Expression trees are immutable. Subgrammars
+        recreate all the Expressions from scratch, and AbstractGrammars have
+        no Expressions.
+
+        """
+        new = Grammar(**self)
+        new.default_rule = self.default_rule
+        return new
+
+    def _expressions_from_rules(self, rules, custom_rules):
         """Return a 2-tuple: a dict of rule names pointing to their
         expressions, and then the first rule.
 
@@ -74,27 +95,43 @@ class Grammar(StrAndRepr, dict):
         starting symbol for parsing, but there's nothing saying you can't have
         multiple roots.
 
+        :arg custom_rules: A map of rule names to custom-coded rules:
+            Expressions
+
         """
         tree = rule_grammar.parse(rules)
-        return RuleVisitor().visit(tree)
+        return RuleVisitor(custom_rules).visit(tree)
 
     def parse(self, text, pos=0):
-        """Parse some text with the default rule."""
-        return self.default_rule.parse(text, pos=pos)
-
-    def match(self, text, pos=0):
-        """Parse some text with the default rule but not necessarily all the
-        way to the end.
+        """Parse some text with the :term:`default rule`.
 
         :arg pos: The index at which to start parsing
 
         """
+        self._check_default_rule()
+        return self.default_rule.parse(text, pos=pos)
+
+    def match(self, text, pos=0):
+        """Parse some text with the :term:`default rule` but not necessarily
+        all the way to the end.
+
+        :arg pos: The index at which to start parsing
+
+        """
+        self._check_default_rule()
         return self.default_rule.match(text, pos=pos)
 
+    def _check_default_rule(self):
+        """Raise RuntimeError if there is no default rule defined."""
+        if not self.default_rule:
+            raise RuntimeError("Can't call parse() on a Grammar that has no "
+                               "default rule. Choose a specific rule instead, "
+                               "like some_grammar['some_rule'].parse(...).")
+
     def __unicode__(self):
         """Return a rule string that, when passed to the constructor, would
         reconstitute the grammar."""
-        exprs = [self.default_rule]
+        exprs = [self.default_rule] if self.default_rule else []
         exprs.extend(expr for expr in self.itervalues() if
                      expr is not self.default_rule)
         return '\n'.join(expr.as_rule() for expr in exprs)
@@ -113,7 +150,7 @@ class BootstrappingGrammar(Grammar):
     grammar description syntax.
 
     """
-    def _expressions_from_rules(self, rule_syntax):
+    def _expressions_from_rules(self, rule_syntax, custom_rules):
         """Return the rules for parsing the grammar definition syntax.
 
         Return a 2-tuple: a dict of rule names pointing to their expressions,
@@ -164,6 +201,7 @@ class BootstrappingGrammar(Grammar):
         # Turn the parse tree into a map of expressions:
         return RuleVisitor().visit(rule_tree)
 
+
 # The grammar for parsing PEG grammar definitions:
 # This is a nice, simple grammar. We may someday add to it, but it's a safe bet
 # that the future will always be a superset of this.
@@ -171,7 +209,7 @@ rule_syntax = (r'''
     # Ignored things (represented by _) are typically hung off the end of the
     # leafmost kinds of nodes. Literals like "/" count as leaves.
 
-    rules = _ rule+
+    rules = _ rule*
     rule = label equals expression
     equals = "=" _
     literal = spaceless_literal _
@@ -229,6 +267,15 @@ class RuleVisitor(NodeVisitor):
 
     visit_expression = visit_term = visit_atom = NodeVisitor.lift_child
 
+    def __init__(self, custom_rules=None):
+        """Construct.
+
+        :arg custom_rules: A dict of {rule name: expression} holding custom
+            rules which will take precedence over the others
+
+        """
+        self.custom_rules = custom_rules or {}
+
     def visit_parenthesized(self, parenthesized, (left_paren, _1,
                                                   expression,
                                                   right_paren, _2)):
@@ -325,58 +372,31 @@ class RuleVisitor(NodeVisitor):
         """
         return visited_children or node  # should semantically be a tuple
 
-    def _resolve_refs(self, rule_map, expr, unwalked_names, walking_names):
+    def _resolve_refs(self, rule_map, expr):
         """Return an expression with all its lazy references recursively
         resolved.
 
         Resolve any lazy references in the expression ``expr``, recursing into
-        all subexpressions. Populate ``rule_map`` with any other rules (named
-        expressions) resolved along the way. Remove from ``unwalked_names`` any
-        which were resolved.
-
-        :arg walking_names: The stack of labels we are currently recursing
-            through. This prevents infinite recursion for circular refs.
+        all subexpressions.
 
         """
-        # If it's a top-level (named) expression and we've already walked it,
-        # don't walk it again:
-        if expr.name and expr.name not in unwalked_names:
-            # unwalked_names started out with all the rule names in it, so, if
-            # this is a named expr and it isn't in there, it must have been
-            # resolved.
-            return rule_map[expr.name]
-
-        # If not, resolve it:
-        elif isinstance(expr, LazyReference):
+        if isinstance(expr, LazyReference):
             label = unicode(expr)
-            if label not in walking_names:
-                # We aren't already working on traversing this label:
-                try:
-                    reffed_expr = rule_map[label]
-                except KeyError:
-                    raise UndefinedLabel(expr)
-                rule_map[label] = self._resolve_refs(
-                        rule_map,
-                        reffed_expr,
-                        unwalked_names,
-                        walking_names + (label,))
-
-                # If we recurse into a compound expression, the remove()
-                # happens in there. But if this label points to a non-compound
-                # expression like a literal or a regex or another lazy
-                # reference, we need to do this here:
-                unwalked_names.discard(label)
-            return rule_map[label]
+            try:
+                reffed_expr = rule_map[label]
+            except KeyError:
+                raise UndefinedLabel(expr)
+            return self._resolve_refs(rule_map, reffed_expr)
         else:
-            members = getattr(expr, 'members', [])
-            if members:
-                expr.members = [self._resolve_refs(rule_map,
-                                                   m,
-                                                   unwalked_names,
-                                                   walking_names)
-                                for m in members]
-            if expr.name:
-                unwalked_names.remove(expr.name)
+            original_members = getattr(expr, 'members', ())
+            if original_members:
+                # Prevents infinite recursion for circular refs. At worst, one
+                # of `expr.members` can refer back to `expr`, but it can't go
+                # any farther.
+                expr.members = ()
+                resolved_members = [self._resolve_refs(rule_map, member)
+                                    for member in original_members]
+                expr.members = resolved_members
             return expr
 
     def visit_rules(self, node, (_, rules)):
@@ -384,25 +404,30 @@ class RuleVisitor(NodeVisitor):
 
         The default rule is the first one. Or, if you have more than one rule
         of that name, it's the last-occurring rule of that name. (This lets you
-        override the default rule when you extend a grammar.)
+        override the default rule when you extend a grammar.) If there are no
+        string-based rules, the default rule is None, because the custom rules,
+        due to being kwarg-based, are unordered.
 
         """
         # Map each rule's name to its Expression. Later rules of the same name
         # override earlier ones. This lets us define rules multiple times and
-        # have the last declarations win, so you can extend grammars by
+        # have the last declaration win, so you can extend grammars by
         # concatenation.
         rule_map = dict((expr.name, expr) for expr in rules)
 
+        # And custom rules override string-based rules. This is the least
+        # surprising choice when you compare the dict constructor:
+        # dict({'x': 5}, x=6).
+        rule_map.update(self.custom_rules)
+
         # Resolve references. This tolerates forward references.
-        unwalked_names = set(rule_map.iterkeys())
-        while unwalked_names:
-            rule_name = next(iter(unwalked_names))  # any arbitrary item
-            rule_map[rule_name] = self._resolve_refs(rule_map,
-                                                     rule_map[rule_name],
-                                                     unwalked_names,
-                                                     (rule_name,))
-            unwalked_names.discard(rule_name)
-        return rule_map, rules[0]
+        rule_map = dict((expr.name, self._resolve_refs(rule_map, expr))
+                        for expr in rule_map.itervalues())
+
+        # isinstance() is a temporary hack around the fact that * rules don't
+        # always get transformed into lists by NodeVisitor. We should fix that;
+        # it's surprising and requires writing lame branches like this.
+        return rule_map, rules[0] if isinstance(rules, list) and rules else None
 
 
 # Bootstrap to level 1...
@@ -413,6 +438,7 @@ rule_grammar = BootstrappingGrammar(rule_syntax)
 # test_grammar.
 rule_grammar = Grammar(rule_syntax)
 
+
 # TODO: Teach Expression trees how to spit out Python representations of
 # themselves. Then we can just paste that in above, and we won't have to
 # bootstrap on import. Though it'll be a little less DRY. [Ah, but this is not
